/* global chrome, localStorage */

var _ = require('lodash')
var Q = require('q')

var chromeStorage = (typeof chrome !== 'undefined' && chrome.storage)

var store = {
  get: function (key) {
    var deferred = Q.defer()
    if (window.localStorage) {
      process.nextTick(function () {
        var value = localStorage.getItem(key)
        try {
          value = JSON.parse(value)
        } catch (e) {
          deferred.reject('JSON parse failed for ' + key)
          return
        }
        if (value === null || value === undefined) {
          deferred.reject('key not found ' + key)
        } else {
          deferred.resolve(value)
        }
      })
    } else if (chromeStorage) {
      chrome.storage.local.get(key, function (items) {
        if (chrome.runtime.lastError) {
          deferred.reject('error while retrieving key ' + key)
        } else if (!_.has(items, key)) {
          deferred.reject('key not found ' + key)
        } else {
          deferred.resolve(items[key])
        }
      })
    }
    return deferred.promise
  },
  put: function (key, value) {
    var deferred = Q.defer()
    if (window.localStorage) {
      process.nextTick(function () {
        value = JSON.stringify(value)
        try {
          localStorage.setItem(key, value)
          deferred.resolve()
        } catch (e) {
          deferred.reject(e)
        }
      })
    } else if (chromeStorage) {
      var items = {}
      items[key] = value
      chrome.storage.local.set(items, function () {
        if (chrome.runtime.lastError) {
          deferred.reject('error while setting key ' + key)
        } else {
          deferred.resolve()
        }
      })
    }
    return deferred.promise
  },
  delete: function (key) {
    var deferred = Q.defer()
    if (window.localStorage) {
      process.nextTick(function () {
        try {
          localStorage.removeItem(key)
          deferred.resolve()
        } catch (e) {
          deferred.reject(e)
        }
      })
    } else if (chromeStorage) {
      chrome.storage.local.remove(key, function () {
        if (chrome.runtime.lastError) {
          deferred.reject()
        } else {
          deferred.resolve()
        }
      })
    }
  }
}

module.exports = store
